@@ -30,7 +30,7 @@ import click
 
 from . import exceptions as E, models
 from .config import Config, setup_logging
-from .resolvers import Defn, Strategies
+from .resolvers import Defn, MultiPkgModel, Strategy
 from .utils import TocReader, cached_property, get_version, is_outdated, tabulate, uniq
 
 if TYPE_CHECKING:
@@ -42,9 +42,9 @@ _EnumT = TypeVar('_EnumT', bound=enum.Enum)
 
 
 class Report:
-    _success_symbol = click.style('✓', fg='green')
-    _failure_symbol = click.style('✗', fg='red')
-    _warning_symbol = click.style('!', fg='blue')
+    SUCCESS_SYMBOL = click.style('✓', fg='green')
+    FAILURE_SYMBOL = click.style('✗', fg='red')
+    WARNING_SYMBOL = click.style('!', fg='blue')
 
     def __init__(
         self,
@@ -57,21 +57,22 @@ class Report:
     @property
     def exit_code(self) -> int:
         return any(
-            r
+            isinstance(r, (E.ManagerError, E.InternalError)) and self.filter_fn(r)
             for _, r in self.results
-            if isinstance(r, (E.ManagerError, E.InternalError)) and self.filter_fn(r)
         )
 
-    def __str__(self) -> str:
-        def adorn_result(result: E.ManagerResult):
-            if isinstance(result, E.InternalError):
-                return self._warning_symbol
-            elif isinstance(result, E.ManagerError):
-                return self._failure_symbol
-            return self._success_symbol
+    @classmethod
+    def _result_type_to_symbol(cls, result: E.ManagerResult) -> str:
+        if isinstance(result, E.InternalError):
+            return cls.WARNING_SYMBOL
+        elif isinstance(result, E.ManagerError):
+            return cls.FAILURE_SYMBOL
+        else:
+            return cls.SUCCESS_SYMBOL
 
+    def __str__(self) -> str:
         return '\n'.join(
-            f'{adorn_result(r)} {click.style(str(a), bold=True)}\n'
+            f'{self._result_type_to_symbol(r)} {click.style(str(a), bold=True)}\n'
             + fill(r.message, initial_indent=' ' * 2, subsequent_indent=' ' * 4)
             for a, r in self.results
             if self.filter_fn(r)
@@ -80,9 +81,9 @@ class Report:
     def generate(self):
         manager: CliManagerT = click.get_current_context().obj.m
         if manager.config.auto_update_check:
-            outdated, new_version = is_outdated()
+            outdated, new_version = manager.run(is_outdated())
             if outdated:
-                click.echo(f'{self._warning_symbol} instawow-{new_version} is available')
+                click.echo(f'{self.WARNING_SYMBOL} instawow-{new_version} is available')
 
         report = str(self)
         if report:
@@ -206,12 +207,11 @@ def parse_into_defn(
             raise click.BadParameter(value)
 
         pair = '*', value
-    source, name = pair
-    return Defn.get(source, name)
+    return Defn(*pair)
 
 
 def parse_into_defn_with_strategy(
-    manager: CliManagerT, value: Sequence[Tuple[Strategies, str]]
+    manager: CliManagerT, value: Sequence[Tuple[Strategy, str]]
 ) -> List[Defn]:
     defns = parse_into_defn(manager, [d for _, d in value])
     return list(map(Defn.with_strategy, defns, (s for s, _ in value)))
@@ -225,7 +225,7 @@ def parse_into_defn_with_version(
 
 
 def parse_into_defn_from_json_file(manager: CliManagerT, path: Path) -> List[Defn]:
-    faux_pkgs = models.MultiPkgModel.parse_file(path, encoding='utf-8')
+    faux_pkgs = MultiPkgModel.parse_file(path, encoding='utf-8')
     return list(map(Defn.from_pkg, cast('List[models.Pkg]', faux_pkgs.__root__)))
 
 
@@ -240,7 +240,7 @@ def combine_addons(
         addons.extend(fn(ctx.obj.m, value))
 
 
-excluded_strategies = {Strategies.default, Strategies.version}
+excluded_strategies = {Strategy.default, Strategy.version}
 
 
 @main.command()
@@ -259,13 +259,13 @@ excluded_strategies = {Strategies.default, Strategies.version}
     '--with-strategy',
     '-s',
     multiple=True,
-    type=(EnumParam(Strategies, excluded_strategies), str),
+    type=(EnumParam(Strategy, excluded_strategies), str),
     expose_value=False,
     callback=partial(combine_addons, parse_into_defn_with_strategy),
     metavar='<STRATEGY ADDON>...',
     help='A strategy followed by an add-on definition.  '
     'The strategies are: '
-    f'{", ".join(s.name for s in Strategies if s not in excluded_strategies)}.',
+    f'{", ".join(s.name for s in Strategy if s not in excluded_strategies)}.',
 )
 @click.option(
     '--version',
@@ -385,7 +385,7 @@ def rollback(ctx: click.Context, addon: Defn, undo: bool) -> None:
 @click.pass_context
 def reconcile(ctx: click.Context, auto: bool, list_unreconciled: bool) -> None:
     "Reconcile pre-installed add-ons."
-    from .matchers import AddonFolder, get_folders, match_dir_names, match_toc_ids, match_toc_names
+    from .matchers import AddonFolder, get_folder_set, match_dir_names, match_toc_ids, match_toc_names
     from .models import is_pkg
     from .prompts import PkgChoice, confirm, select, skip
 
@@ -434,10 +434,7 @@ def reconcile(ctx: click.Context, auto: bool, list_unreconciled: bool) -> None:
         for addons, defns in groups:
             shortlist = list(filter(is_pkg, (results[d] for d in defns)))
             if shortlist:
-                if auto:
-                    selection = Defn.from_pkg(shortlist[0])
-                else:
-                    selection = prompt_one(addons, shortlist)
+                selection = Defn.from_pkg(shortlist[0]) if auto else prompt_one(addons, shortlist)
                 selection and (yield selection)
 
     def match_all() -> Generator[List[Defn], FrozenSet[AddonFolder], None]:
@@ -450,7 +447,7 @@ def reconcile(ctx: click.Context, auto: bool, list_unreconciled: bool) -> None:
             groups = manager.run(fn(manager, (yield [])))
             yield list(prompt(groups))
 
-    leftovers = get_folders(manager)
+    leftovers = get_folder_set(manager)
     if list_unreconciled:
         table_rows = [('unreconciled',), *((f.name,) for f in sorted(leftovers))]
         click.echo(tabulate(table_rows))
@@ -469,7 +466,7 @@ def reconcile(ctx: click.Context, auto: bool, list_unreconciled: bool) -> None:
             results = manager.run(manager.install(selections, replace=True))
             Report(results.items()).generate()
 
-        leftovers = get_folders(manager)
+        leftovers = get_folder_set(manager)
 
     if leftovers:
         click.echo()
@@ -528,7 +525,7 @@ def list_installed(
     from sqlalchemy import and_, or_
 
     def format_deps(pkg: models.Pkg):
-        deps = (Defn.get(pkg.source, d.id) for d in pkg.deps)
+        deps = (Defn(pkg.source, d.id) for d in pkg.deps)
         deps = (d.with_(name=getattr(obj.m.get_pkg(d), 'slug', d.alias)) for d in deps)
         return map(str, deps)
 
@@ -558,7 +555,7 @@ def list_installed(
         .all()
     )
     if output_format is ListFormats.json:
-        click.echo(models.MultiPkgModel.parse_obj(pkgs).json(indent=2))
+        click.echo(MultiPkgModel.parse_obj(pkgs).json(indent=2))
     elif output_format is ListFormats.detailed:
         formatter = click.HelpFormatter(max_width=99)
         for pkg in pkgs:
@@ -627,9 +624,6 @@ def configure(ctx: click.Context) -> Config:
 
     from .prompts import PydanticValidator
 
-    profile = ctx.find_root().params['profile']
-    game_flavours = Config.__fields__['game_flavour'].type_.__args__
-
     addon_dir = text(
         'Add-on directory: ',
         completer=PathCompleter(only_directories=True, expanduser=True),
@@ -637,11 +631,13 @@ def configure(ctx: click.Context) -> Config:
     ).unsafe_ask()
     game_flavour = text(
         'Game flavour: ',
-        default='classic' if '_classic_' in addon_dir else 'retail',
-        completer=WordCompleter(game_flavours),
+        default='classic' if Config.is_classic_folder(addon_dir) else 'retail',
+        completer=WordCompleter(['retail', 'classic']),
         validate=PydanticValidator(Config, 'game_flavour'),
     ).unsafe_ask()
-    config = Config(profile=profile, addon_dir=addon_dir, game_flavour=game_flavour).write()
+    config = Config(
+        profile=ctx.find_root().params['profile'], addon_dir=addon_dir, game_flavour=game_flavour
+    ).write()
     click.echo(f'Configuration written to: {config.config_file}')
     return config
 
@@ -652,41 +648,29 @@ def _weakauras_group():
 
 
 @_weakauras_group.command('build')
-@click.option(
-    '--account',
-    '-a',
-    required=True,
-    help='Your account name.  This is used to locate the WeakAuras data file.',
-)
 @click.pass_obj
-def build_weakauras_companion(obj: ManagerWrapper, account: str) -> None:
+def build_weakauras_companion(obj: ManagerWrapper) -> None:
     "Build the WeakAuras Companion add-on."
     from .wa_updater import BuilderConfig, WaCompanionBuilder
 
-    config = BuilderConfig(account=account)
+    config = BuilderConfig()
     obj.m.run(WaCompanionBuilder(obj.m, config).build())
 
 
 @_weakauras_group.command('list')
-@click.option(
-    '--account',
-    '-a',
-    required=True,
-    help='Your account name.  This is used to locate the WeakAuras data file.',
-)
 @click.pass_obj
-def list_installed_wago_auras(obj: ManagerWrapper, account: str) -> None:
+def list_installed_wago_auras(obj: ManagerWrapper) -> None:
     "List WeakAuras installed from Wago."
     from textwrap import fill
 
     from .wa_updater import BuilderConfig, WaCompanionBuilder
 
-    config = BuilderConfig(account=account)
+    config = BuilderConfig()
     aura_groups = WaCompanionBuilder(obj.m, config).extract_installed_auras()
     installed_auras = sorted(
-        (g.filename, fill(a.id, width=30, max_lines=1), a.url)
+        (g._filename, fill(a.id, width=30, max_lines=1), a.url)
         for g in aura_groups
-        for v in g.entries.values()
+        for v in g.__root__.values()
         for a in v
         if not a.parent
     )
@@ -705,9 +689,9 @@ def generate_catalogue(filename: Path, age_cutoff: O[datetime]) -> None:
     "Generate the master catalogue."
     import asyncio
 
-    from .resolvers import MasterCatalogue
+    from .resolvers import Catalogue
 
-    catalogue = asyncio.run(MasterCatalogue.collate(age_cutoff))
+    catalogue = asyncio.run(Catalogue.collate(age_cutoff))
     filename.write_text(catalogue.json(indent=2), encoding='utf-8')
     filename.with_suffix(f'.compact{filename.suffix}').write_text(
         catalogue.json(separators=(',', ':')), encoding='utf-8'
