@@ -23,18 +23,20 @@ from typing import (
     Union,
     overload,
 )
+from uuid import uuid4
 
 from aiohttp import web
 from aiohttp_rpc import JsonRpcMethod, WsJsonRpcServer, middlewares as rpc_middlewares
 from aiohttp_rpc.errors import InvalidParams as InvalidParamsError, ServerError
 from pydantic import BaseModel, ValidationError, validator
+from yarl import URL
 
 from . import exceptions as E
 from .config import Config
 from .manager import Manager, init_web_client
-from .matchers import get_folders, match_dir_names, match_toc_ids, match_toc_names
-from .models import Pkg, PkgModel, is_pkg
-from .resolvers import Defn, Strategies
+from .matchers import get_folder_set, match_dir_names, match_toc_ids, match_toc_names
+from .models import Pkg, is_pkg
+from .resolvers import Defn, PkgModel, Strategy
 from .utils import Literal, get_version, is_outdated, run_in_thread as t, uniq
 
 if TYPE_CHECKING:
@@ -42,6 +44,7 @@ if TYPE_CHECKING:
     ManagerWorkQueueItem = Tuple[asyncio.Future[Any], str, O[Callable[..., Awaitable[Any]]]]
 
 
+LOCALHOST = '127.0.0.1'
 API_VERSION = 0
 
 
@@ -119,12 +122,12 @@ class ListProfilesParams(BaseParams):
 class Source(BaseModel):
     source: str
     name: str
-    supported_strategies: List[Strategies]
+    supported_strategies: List[Strategy]
     supports_rollback: bool
 
     @validator('supported_strategies')
-    def _sort_strategies(cls, value: List[Strategies]) -> List[Strategies]:
-        return sorted(value, key=list(Strategies).index)
+    def _sort_strategies(cls, value: List[Strategy]) -> List[Strategy]:
+        return sorted(value, key=list(Strategy).index)
 
 
 class ListSourcesParams(_ProfileParamMixin, BaseParams):
@@ -182,7 +185,7 @@ class MultiResult(BaseModel):
 class SearchParams(_ProfileParamMixin, BaseParams):
     search_terms: str
     limit: int
-    strategy: Strategies = Strategies.default
+    strategy: Strategy = Strategy.default
     _method = 'search'
 
     async def respond(self, managers: ManagerWorkQueue) -> MultiResult:
@@ -317,7 +320,7 @@ class ReconcileParams(_ProfileParamMixin, BaseParams):
     _method = 'reconcile'
 
     async def respond(self, managers: ManagerWorkQueue) -> ReconcileResult:
-        leftovers = await managers.run(self.profile, t(get_folders))
+        leftovers = await managers.run(self.profile, t(get_folder_set))
         match_groups = await managers.run(
             self.profile, partial(_matchers[self.matcher], leftovers=leftovers)
         )
@@ -343,7 +346,7 @@ class GetVersionParams(BaseParams):
     _method = 'meta/get_version'
 
     async def respond(self, managers: ManagerWorkQueue) -> GetVersionResult:
-        outdated, new_version = await t(is_outdated)()
+        outdated, new_version = await is_outdated()
         return GetVersionResult(
             installed_version=get_version(), new_version=new_version if outdated else None
         )
@@ -429,8 +432,19 @@ def serialise_response(value: Dict[str, Any]) -> str:
     return BaseModel.construct(**value).json()
 
 
-async def create_app() -> web.Application:
+async def create_app() -> Tuple[web.Application, str]:
     managers = ManagerWorkQueue()
+
+    def start_managers():
+        managers_listen = asyncio.create_task(managers.listen())
+
+        async def on_shutdown(app: web.Application):
+            managers_listen.cancel()
+            await managers.cleanup()
+
+        return on_shutdown
+
+    endpoint = f'/v{API_VERSION}/{uuid4()}'
     rpc_server = WsJsonRpcServer(
         json_serialize=serialise_response,
         middlewares=rpc_middlewares.DEFAULT_MIDDLEWARES,
@@ -458,33 +472,28 @@ async def create_app() -> web.Application:
         )
     )
     app = web.Application()
-    app.router.add_routes([web.get(f'/v{API_VERSION}', rpc_server.handle_http_request)])
-
-    async def on_shutdown(app: web.Application):
-        listen.cancel()
-        await managers.cleanup()
-
-    listen = asyncio.create_task(managers.listen())
-    app.on_shutdown.append(on_shutdown)
-    return app
+    app.add_routes([web.get(endpoint, rpc_server.handle_http_request)])
+    app.on_shutdown.append(start_managers())
+    app.freeze()
+    return (app, endpoint)
 
 
 async def listen() -> None:
     "Fire up the server."
     loop = asyncio.get_running_loop()
-    app_runner = web.AppRunner(await create_app())
+    app, endpoint = await create_app()
+    app_runner = web.AppRunner(app)
     await app_runner.setup()
     # Placate the type checker - server is created in ``app_runner.setup``
     assert app_runner.server
-    # By omitting the port ``loop.create_server`` will find a random available
-    # port to bind to (equivalent to creating a socket on port 0)
-    server = await loop.create_server(app_runner.server, '0.0.0.0')
+    # By omitting the port, ``loop.create_server`` will find an available port
+    # to bind to - this is equivalent to creating a socket on port 0.
+    server = await loop.create_server(app_runner.server, LOCALHOST)
     assert server.sockets
     (host, port) = server.sockets[0].getsockname()
     # We're writing the address to fd 3 just in case something seeps into
     # stdout or stderr.
-    # The message needs to be in JSON for quote, unquote IPC with Electron
-    message = f'{{"address": "ws://{host}:{port}/"}}\n'.encode()
+    message = str(URL.build(scheme='ws', host=host, port=port, path=endpoint)).encode()
     for fd in (1, 3):
         os.write(fd, message)
     try:
