@@ -8,30 +8,34 @@ from typing import (
     TYPE_CHECKING,
     Any,
     AsyncIterable,
+    Callable,
     ClassVar,
     Dict,
+    FrozenSet,
     List,
     Optional as O,
     Sequence,
     Set,
+    Tuple,
     Union,
     cast,
 )
 
-from pydantic import BaseModel, WrongConstantError, validator
-from typing_extensions import Literal, TypedDict, get_args
+from pydantic import BaseModel
+from pydantic.datetime_parse import parse_datetime
+from typing_extensions import Literal, TypedDict
 from yarl import URL
 
 from . import exceptions as E, models as m
-from .utils import bucketise, cached_property, gather, run_in_thread as t, slugify, uniq
+from .utils import bucketise, cached_property, gather, run_in_thread as t, uniq
 
 if TYPE_CHECKING:
-    from pydantic.fields import ModelField as ModeFieldT
+    import aiohttp
 
     from .manager import Manager as ManagerT
 
 
-class Strategies(enum.Enum):
+class Strategy(enum.Enum):
     default = 'default'
     latest = 'latest'
     curse_latest_beta = 'curse_latest_beta'
@@ -40,90 +44,64 @@ class Strategies(enum.Enum):
     version = 'version'
 
 
-class HashableModel(BaseModel):
+class _HashableModel(BaseModel):
     class Config:
         allow_mutation = False
 
     def __eq__(self, other: Any) -> bool:
-        if not isinstance(other, self.__class__):
-            return NotImplemented
-        return self.__dict__ == other.__dict__
+        return self.__dict__ == (other.__dict__ if isinstance(other, self.__class__) else other)
 
     def __hash__(self) -> int:
         return hash(tuple(self.__dict__.items()))
 
 
-class BaseStrategy(HashableModel):
-    @validator('type_', check_fields=False, pre=True)
-    def _parse_strategy(cls, value: Any, field: ModeFieldT) -> Strategies:
-        accepted_strategies = get_args(field.type_)
-        strategy = Strategies.__members__.get(value) or value
-        if strategy in accepted_strategies:
-            return strategy
-        raise WrongConstantError(given=value, permitted=accepted_strategies)
-
-
-class SimpleStrategy(BaseStrategy):
-    type_: Literal[
-        Strategies.default,
-        Strategies.latest,
-        Strategies.curse_latest_beta,
-        Strategies.curse_latest_alpha,
-        Strategies.any_flavour,
-    ]
-
-
-class VersionStrategy(BaseStrategy):
-    type_: Literal[Strategies.version]
-    version: str
-
-
-class Defn(HashableModel):
+class Defn(_HashableModel):
     source: str
-    id: O[str] = None
     alias: str
-    strategy: Union[SimpleStrategy, VersionStrategy] = SimpleStrategy(type_=Strategies.default)
+    id: O[str] = None
+    strategy: Strategy = Strategy.default
+    version: O[str] = None
 
     @classmethod
     def from_pkg(cls, pkg: m.Pkg) -> Defn:
-        strategy_kwargs: Dict[str, str] = (
-            {'version': pkg.version} if pkg.options.strategy == 'version' else {}
-        )
         return cls(
             source=pkg.source,
             id=pkg.id,
             alias=pkg.slug,
-            strategy={'type_': pkg.options.strategy, **strategy_kwargs},
+            strategy=pkg.options.strategy,
+            version=pkg.version,
         )
 
-    @classmethod
-    def get(cls, source: str, alias: str) -> Defn:
-        return cls(source=source, alias=alias)
+    def __init__(self, source: str, alias: str, **kwargs: Any) -> None:
+        super().__init__(source=source, alias=alias, **kwargs)
 
     def with_(self, **kwargs: Any) -> Defn:
         return self.__class__(**{**self.__dict__, **kwargs})
 
-    def with_strategy(self, strategy: Strategies) -> Defn:
-        return self.with_(strategy={'type_': strategy})
+    def with_strategy(self, strategy: Strategy) -> Defn:
+        return self.with_(strategy=strategy)
 
     def with_version(self, version: str) -> Defn:
-        return self.with_(strategy={'type_': Strategies.version, 'version': version})
+        return self.with_(strategy=Strategy.version, version=version)
 
     def __str__(self) -> str:
         return f'{self.source}:{self.alias}'
 
 
-def normalise_names():
+def normalise_names(replace_delim: str = ' '):
     import string
 
     trans_table = str.maketrans(dict.fromkeys(string.punctuation, ' '))
 
     def normalise(value: str):
-        return ' '.join(value.casefold().translate(trans_table).split())
+        return replace_delim.join(value.casefold().translate(trans_table).split())
 
     return normalise
 
 
+_slugify = normalise_names('-')
+
+
 class _CatalogueEntryDefaultFields(TypedDict):
     source: str
     id: str
@@ -141,30 +119,31 @@ class _CatalogueEntry(BaseModel):
     slug: str
     name: str
     game_compatibility: Set[Literal['retail', 'classic']]
-    folders: List[List[str]]
+    folders: List[FrozenSet[str]]
     download_count: int
     last_updated: datetime
     normalised_name: str
     derived_download_score: float
 
 
-class MasterCatalogue(BaseModel):
+class Catalogue(BaseModel):
     __root__: List[_CatalogueEntry]
 
     class Config:
-        keep_untouched = (cast(Any, cached_property),)
+        json_encoders: Dict[type, Callable[..., Any]] = {
+            set: sorted,
+            frozenset: sorted,
+        }
+        keep_untouched: Tuple[type, ...] = (cached_property,)
 
     @classmethod
-    async def collate(cls, age_cutoff: O[datetime]) -> MasterCatalogue:
-        from types import SimpleNamespace
-
+    async def collate(cls, age_cutoff: O[datetime]) -> Catalogue:
         from .manager import init_web_client
 
-        resolvers = (CurseResolver, WowiResolver, TukuiResolver)
+        resolvers = (CurseResolver, WowiResolver, TukuiResolver, InstawowResolver)
 
         async with init_web_client() as web_client:
-            faux_manager = cast('ManagerT', SimpleNamespace(web_client=web_client))
-            items = [a for r in resolvers async for a in r(faux_manager).collect_items()]
+            items = [a for r in resolvers async for a in r.collect_items(web_client)]
 
         normalise = normalise_names()
         most_downloads_per_source = {
@@ -190,27 +169,61 @@ class MasterCatalogue(BaseModel):
         return {a.slug: a.id for a in self.__root__ if a.source == 'curse'}
 
 
+class _OrmModel(BaseModel):
+    class Config:
+        allow_mutation = False
+        orm_mode = True
+
+
+class _PkgModel_PkgFolder(_OrmModel):
+    name: str
+
+
+class _PkgModel_PkgOptions(_OrmModel):
+    strategy: Strategy
+
+
+class _PkgModel_PkgDep(_OrmModel):
+    id: str
+
+
+class _PkgModel_PkgVersion(_OrmModel):
+    version: str
+    install_time: datetime
+
+
+class PkgModel(_OrmModel):
+    source: str
+    id: str
+    slug: str
+    name: str
+    description: str
+    url: str
+    download_url: str
+    date_published: datetime
+    version: str
+    folders: List[_PkgModel_PkgFolder]
+    options: _PkgModel_PkgOptions
+    deps: List[_PkgModel_PkgDep]
+    logged_versions: List[_PkgModel_PkgVersion]
+
+
+class MultiPkgModel(BaseModel):
+    __root__: List[PkgModel]
+
+
 class Resolver:
     source: ClassVar[str]
     name: ClassVar[str]
-    strategies: ClassVar[Set[Strategies]]
+    strategies: ClassVar[Set[Strategy]]
 
     def __init__(self, manager: ManagerT) -> None:
         self.manager = manager
 
-    def __init_subclass__(cls) -> None:
-        async def resolve_wrapper(self: Resolver, defn: Defn, metadata: O[Any]) -> m.Pkg:
-            if defn.strategy.type_ in self.strategies:
-                return await resolve_one(self, defn, metadata)  # type: ignore
-            raise E.PkgStrategyUnsupported(defn.strategy.type_)
-
-        resolve_one = cls.resolve_one
-        cls.resolve_one = resolve_wrapper  # type: ignore
-
     @property
     def supports_rollback(self) -> bool:
         "Whether the resolver supports rollback operations."
-        return Strategies.version in self.strategies
+        return Strategy.version in self.strategies
 
     @staticmethod
     def get_alias_from_url(value: str) -> O[str]:
@@ -225,7 +238,10 @@ class Resolver:
         "Resolve an individual definition into a package."
         raise NotImplementedError
 
-    async def collect_items(self) -> AsyncIterable[_CatalogueEntryDefaultFields]:
+    @classmethod
+    async def collect_items(
+        cls, web_client: aiohttp.ClientSession
+    ) -> AsyncIterable[_CatalogueEntryDefaultFields]:
         "Yield add-ons from source for cataloguing."
         return
         yield
@@ -234,7 +250,7 @@ class Resolver:
 if TYPE_CHECKING:
 
     # Only documenting the fields we're actually using -
-    # the API responses are absolutely massive
+    # the API response is absolutely massive
 
     class CurseAddon_FileDependency(TypedDict):
         id: int  # Unique dependency ID
@@ -251,7 +267,15 @@ if TYPE_CHECKING:
 
     class CurseAddon_FileModules(TypedDict):
         foldername: str
-        fingerprint: int
+        fingerprint: int  # Folder fingerprint used by Curse for reconciliation
+        # One of:
+        #   1 = package
+        #   2 = module
+        #   3 = main module
+        #   4 = file
+        #   5 = referenced file
+        # For WoW add-ons the main folder will have type "3" and the rest
+        # of them type "2"
         type: int
 
     class CurseAddon_File(TypedDict):
@@ -281,12 +305,12 @@ class CurseResolver(Resolver):
     source = 'curse'
     name = 'CurseForge'
     strategies = {
-        Strategies.default,
-        Strategies.latest,
-        Strategies.curse_latest_beta,
-        Strategies.curse_latest_alpha,
-        Strategies.any_flavour,
-        Strategies.version,
+        Strategy.default,
+        Strategy.latest,
+        Strategy.curse_latest_beta,
+        Strategy.curse_latest_alpha,
+        Strategy.any_flavour,
+        Strategy.version,
     }
 
     # Reference: https://twitchappapi.docs.apiary.io/
@@ -307,14 +331,14 @@ class CurseResolver(Resolver):
     async def resolve(self, defns: Sequence[Defn]) -> Dict[Defn, Any]:
         from aiohttp import ClientResponseError
 
-        from .manager import cache_json_response
+        from .manager import cache_response
 
-        defns_to_ids = {
-            d: (d.id or self.manager.catalogue.curse_slugs.get(d.alias) or d.alias) for d in defns
-        }
+        catalogue = await self.manager.synchronise()
+
+        defns_to_ids = {d: (d.id or catalogue.curse_slugs.get(d.alias) or d.alias) for d in defns}
         numeric_ids = {i for i in defns_to_ids.values() if i.isdigit()}
         try:
-            json_response = await cache_json_response(
+            json_response = await cache_response(
                 self.manager,
                 self.addon_api_url,
                 300,
@@ -325,7 +349,7 @@ class CurseResolver(Resolver):
                 raise
             json_response = []
 
-        api_results = {str(r['id']): r for r in json_response}
+        api_results: Dict[str, CurseAddon] = {str(r['id']): r for r in json_response}
         results = await gather(
             self.resolve_one(d, api_results.get(i)) for d, i in defns_to_ids.items()
         )
@@ -335,19 +359,22 @@ class CurseResolver(Resolver):
         if not metadata:
             raise E.PkgNonexistent
 
-        if defn.strategy.type_ is Strategies.version:
+        if defn.strategy not in self.strategies:
+            raise E.PkgStrategyUnsupported(defn.strategy)
 
-            from .manager import cache_json_response
+        if defn.strategy is Strategy.version:
 
-            files = await cache_json_response(
+            from .manager import cache_response
+
+            files = await cache_response(
                 self.manager,
                 self.addon_api_url / str(metadata['id']) / 'files',
                 3600,
                 label=f'Fetching metadata from {self.name}',
             )
 
-            def is_compatible(f: CurseAddon_File):
-                return cast(VersionStrategy, defn.strategy).version == f['displayName']
+            def is_match(f: CurseAddon_File):
+                return defn.version == f['displayName']
 
         else:
 
@@ -359,7 +386,7 @@ class CurseResolver(Resolver):
                 # absent from the /file endpoint
                 return not f['exposeAsAlternative']
 
-            if defn.strategy.type_ is Strategies.any_flavour:
+            if defn.strategy is Strategy.any_flavour:
 
                 def supports_game_version(f: CurseAddon_File):
                     return True
@@ -379,17 +406,17 @@ class CurseResolver(Resolver):
                         for v in f['gameVersion']
                     )
 
-            if defn.strategy.type_ is Strategies.latest:
+            if defn.strategy is Strategy.latest:
 
                 def has_release_type(f: CurseAddon_File):
                     return True
 
-            elif defn.strategy.type_ is Strategies.curse_latest_beta:
+            elif defn.strategy is Strategy.curse_latest_beta:
 
                 def has_release_type(f: CurseAddon_File):
                     return f['releaseType'] == 2
 
-            elif defn.strategy.type_ is Strategies.curse_latest_alpha:
+            elif defn.strategy is Strategy.curse_latest_alpha:
 
                 def has_release_type(f: CurseAddon_File):
                     return f['releaseType'] == 3
@@ -399,42 +426,45 @@ class CurseResolver(Resolver):
                 def has_release_type(f: CurseAddon_File):
                     return f['releaseType'] == 1
 
-            def is_compatible(f: CurseAddon_File):
+            def is_match(f: CurseAddon_File):
                 return is_not_libless(f) and supports_game_version(f) and has_release_type(f)
 
         if not files:
             raise E.PkgFileUnavailable('no files available for download')
         try:
             file = max(
-                filter(is_compatible, files),
+                filter(is_match, files),
                 # The ``id`` is just a counter so we don't have to go digging around dates
                 key=lambda f: f['id'],
             )
         except ValueError:
             raise E.PkgFileUnavailable(
                 f'no files compatible with {self.manager.config.game_flavour} '
-                f'using {defn.strategy.type_.name!r} strategy'
+                f'using {defn.strategy.name!r} strategy'
             )
 
         return m.Pkg(
             source=self.source,
-            id=metadata['id'],
+            id=str(metadata['id']),
             slug=metadata['slug'],
             name=metadata['name'],
             description=metadata['summary'],
             url=metadata['websiteUrl'],
             download_url=file['downloadUrl'],
-            date_published=file['fileDate'],
+            date_published=parse_datetime(file['fileDate']),
             version=file['displayName'],
-            options=m.PkgOptions(strategy=defn.strategy.type_.name),
-            deps=[m.PkgDep(id=d['addonId']) for d in file['dependencies'] if d['type'] == 3],
+            options=m.PkgOptions(strategy=defn.strategy.name),
+            deps=[m.PkgDep(id=str(d['addonId'])) for d in file['dependencies'] if d['type'] == 3],
         )
 
-    async def collect_items(self) -> AsyncIterable[_CatalogueEntryDefaultFields]:
+    @classmethod
+    async def collect_items(
+        cls, web_client: aiohttp.ClientSession
+    ) -> AsyncIterable[_CatalogueEntryDefaultFields]:
         classic_version_prefix = '1.13'
         flavours = ('retail', 'classic')
 
-        def excise_compatibility(files: List[CurseAddon_File]):
+        def excise_flavours(files: List[CurseAddon_File]):
             for c in flavours:
                 if any(f['gameVersionFlavor'] == f'wow_{c}' for f in files):
                     yield c
@@ -448,14 +478,16 @@ class CurseResolver(Resolver):
         step = 1000
         sort_order = '3'  # Alphabetical
         for index in count(0, step):
-            url = (self.addon_api_url / 'search').with_query(
-                gameId='1', sort=sort_order, pageSize=step, index=index
-            )
-            async with self.manager.web_client.get(url) as response:
+            async with web_client.get(
+                (cls.addon_api_url / 'search').with_query(
+                    gameId='1', sort=sort_order, pageSize=step, index=index
+                )
+            ) as response:
                 json_response: List[CurseAddon] = await response.json()
 
             if not json_response:
                 break
+
             for item in json_response:
                 folders = uniq(
                     tuple(m['foldername'] for m in f['modules'])
@@ -463,11 +495,11 @@ class CurseResolver(Resolver):
                     if not f['exposeAsAlternative']
                 )
                 yield _CatalogueEntryDefaultFields(
-                    source=self.source,
+                    source=cls.source,
                     id=str(item['id']),
                     slug=item['slug'],
                     name=item['name'],
-                    game_compatibility=set(excise_compatibility(item['latestFiles'])),
+                    game_compatibility=set(excise_flavours(item['latestFiles'])),
                     folders=folders,
                     download_count=item['downloadCount'],
                     last_updated=item['dateReleased'],
@@ -521,7 +553,7 @@ if TYPE_CHECKING:
 class WowiResolver(Resolver):
     source = 'wowi'
     name = 'WoWInterface'
-    strategies = {Strategies.default}
+    strategies = {Strategy.default}
 
     # Reference: https://api.mmoui.com/v3/globalconfig.json
     # There's also a v4 API corresponding to the as yet unreleased Minion v4,
@@ -563,11 +595,11 @@ class WowiResolver(Resolver):
     async def resolve(self, defns: Sequence[Defn]) -> Dict[Defn, Any]:
         from aiohttp import ClientResponseError
 
-        from .manager import cache_json_response
+        from .manager import cache_response
 
         async with self.manager.locks['load WoWI catalogue']:
             if self.list_api_items is None:
-                list_api_items = await cache_json_response(
+                list_api_items = await cache_response(
                     self.manager,
                     self.list_api_url,
                     3600,
@@ -577,17 +609,18 @@ class WowiResolver(Resolver):
 
         defns_to_ids = {d: ''.join(takewhile(str.isdigit, d.alias)) for d in defns}
         numeric_ids = {i for i in defns_to_ids.values() if i.isdigit()}
-        url = self.details_api_url / f'{",".join(numeric_ids)}.json'
         try:
-            details_api_items = await cache_json_response(self.manager, url, 300)
+            details_api_items = await cache_response(
+                self.manager, self.details_api_url / f'{",".join(numeric_ids)}.json', 300
+            )
         except ClientResponseError as error:
             if error.status != 404:
                 raise
             details_api_items = []
 
         combined_items = {
-            r['UID']: cast('WowiCombinedItem', {**self.list_api_items[r['UID']], **r})
-            for r in details_api_items
+            i['UID']: cast('WowiCombinedItem', {**self.list_api_items[i['UID']], **i})
+            for i in details_api_items
         }
         results = await gather(
             self.resolve_one(d, combined_items.get(i)) for d, i in defns_to_ids.items()
@@ -598,26 +631,32 @@ class WowiResolver(Resolver):
         if not metadata:
             raise E.PkgNonexistent
 
+        if defn.strategy not in self.strategies:
+            raise E.PkgStrategyUnsupported(defn.strategy)
+
         return m.Pkg(
             source=self.source,
             id=metadata['UID'],
-            slug=slugify(f'{metadata["UID"]} {metadata["UIName"]}'),
+            slug=_slugify(f'{metadata["UID"]} {metadata["UIName"]}'),
             name=metadata['UIName'],
             description=metadata['UIDescription'],
             url=metadata['UIFileInfoURL'],
             download_url=metadata['UIDownload'],
-            date_published=metadata['UIDate'],
+            date_published=parse_datetime(metadata['UIDate']),
             version=metadata['UIVersion'],
-            options=m.PkgOptions(strategy=defn.strategy.type_.name),
+            options=m.PkgOptions(strategy=defn.strategy.name),
         )
 
-    async def collect_items(self) -> AsyncIterable[_CatalogueEntryDefaultFields]:
-        async with self.manager.web_client.get(self.list_api_url) as response:
+    @classmethod
+    async def collect_items(
+        cls, web_client: aiohttp.ClientSession
+    ) -> AsyncIterable[_CatalogueEntryDefaultFields]:
+        async with web_client.get(cls.list_api_url) as response:
             list_api_items: List[WowiListApiItem] = await response.json()
 
         for list_item in list_api_items:
             yield _CatalogueEntryDefaultFields(
-                source=self.source,
+                source=cls.source,
                 id=list_item['UID'],
                 name=list_item['UIName'],
                 slug='',
@@ -639,7 +678,7 @@ if TYPE_CHECKING:
         git: str
         id: Literal[-1, -2]  # -1 is Tukui and -2 ElvUI
         lastdownload: str
-        lastupdate: str  # ISO date, e.g. '2020-02-02'
+        lastupdate: str  # ISO date and no tz, e.g. '2020-02-02'
         name: str
         patch: str
         screenshot_url: str
@@ -671,7 +710,7 @@ if TYPE_CHECKING:
 class TukuiResolver(Resolver):
     source = 'tukui'
     name = 'Tukui'
-    strategies = {Strategies.default}
+    strategies = {Strategy.default}
 
     # There's also a ``/client-api.php`` endpoint which is apparently
     # used by the Tukui client itself to check for updates for the two retail
@@ -693,6 +732,9 @@ class TukuiResolver(Resolver):
                 return alias
 
     async def resolve_one(self, defn: Defn, metadata: Any) -> m.Pkg:
+        if defn.strategy not in self.strategies:
+            raise E.PkgStrategyUnsupported(defn.strategy)
+
         alias = ui_id = self.retail_uis.get(defn.alias)
         if not alias:
             alias = ''.join(takewhile('-'.__ne__, defn.alias))
@@ -713,42 +755,50 @@ class TukuiResolver(Resolver):
 
         return m.Pkg(
             source=self.source,
-            id=addon['id'],
-            slug=ui_id or slugify(f'{addon["id"]} {addon["name"]}'),
+            id=str(addon['id']),
+            slug=ui_id or _slugify(f'{addon["id"]} {addon["name"]}'),
             name=addon['name'],
             description=addon['small_desc'],
             url=addon['web_url'],
             download_url=addon['url'],
-            date_published=datetime.fromisoformat(addon['lastupdate']).astimezone(timezone.utc),
+            date_published=datetime.fromisoformat(addon['lastupdate']).replace(
+                tzinfo=timezone.utc
+            ),
             version=addon['version'],
-            options=m.PkgOptions(strategy=defn.strategy.type_.name),
+            options=m.PkgOptions(strategy=defn.strategy.name),
         )
 
-    async def collect_items(self) -> AsyncIterable[_CatalogueEntryDefaultFields]:
-        for query, param, game_compatibility in [
-            ('ui', 'tukui', 'retail'),
-            ('ui', 'elvui', 'retail'),
-            ('addons', 'all', 'retail'),
-            ('classic-addons', 'all', 'classic'),
+    @classmethod
+    async def collect_items(
+        cls, web_client: aiohttp.ClientSession
+    ) -> AsyncIterable[_CatalogueEntryDefaultFields]:
+        for query, param in [
+            ('ui', 'tukui'),
+            ('ui', 'elvui'),
+            ('addons', 'all'),
+            ('classic-addons', 'all'),
         ]:
-            async with self.manager.web_client.get(
-                self.api_url.with_query({query: param})
-            ) as response:
+            async with web_client.get(cls.api_url.with_query({query: param})) as response:
                 metadata = await response.json(content_type=None)  # text/html
 
-            for item in cast(
-                'List[Union[TukuiUi, TukuiAddon]]', [metadata] if query == 'ui' else metadata
-            ):
+            items: List[Union[TukuiUi, TukuiAddon]] = [metadata] if query == 'ui' else metadata
+            game_compatibility = 'classic' if query == 'classic-addons' else 'retail'
+            for item in items:
                 yield _CatalogueEntryDefaultFields(
-                    source=self.source,
+                    source=cls.source,
                     id=str(item['id']),
                     slug='',
                     name=item['name'],
                     folders=[],
                     game_compatibility={game_compatibility},
-                    download_count=int(item['downloads']),
-                    last_updated=datetime.fromisoformat(item['lastupdate']).astimezone(
-                        timezone.utc
+                    # Split Tukui and ElvUI downloads evenly between them.
+                    # They both have the exact same number of downloads so
+                    # I'm assuming they're being counted together.
+                    # Anyway, this should help with scoring other add-ons
+                    # on the Tukui catalogue higher
+                    download_count=int(item['downloads']) // (2 if query == 'ui' else 1),
+                    last_updated=datetime.fromisoformat(item['lastupdate']).replace(
+                        tzinfo=timezone.utc
                     ),
                 )
 
@@ -780,8 +830,8 @@ class GithubResolver(Resolver):
     source = 'github'
     name = 'GitHub'
     strategies = {
-        Strategies.default,
-        Strategies.version,
+        Strategy.default,
+        Strategy.version,
     }
 
     repos_api_url = URL('https://api.github.com/repos')
@@ -808,18 +858,21 @@ class GithubResolver(Resolver):
         """
         from aiohttp import ClientResponseError
 
-        from .manager import cache_json_response
+        from .manager import cache_response
+
+        if defn.strategy not in self.strategies:
+            raise E.PkgStrategyUnsupported(defn.strategy)
 
         repo_url = self.repos_api_url / defn.alias
         try:
-            project_metadata: GithubRepo = await cache_json_response(self.manager, repo_url, 3600)
+            project_metadata: GithubRepo = await cache_response(self.manager, repo_url, 3600)
         except ClientResponseError as error:
             if error.status == 404:
                 raise E.PkgNonexistent
             raise
 
-        if defn.strategy.type_ is Strategies.version:
-            release_url = repo_url / 'releases/tags' / cast(VersionStrategy, defn.strategy).version
+        if defn.strategy is Strategy.version:
+            release_url = repo_url / 'releases/tags' / cast(str, defn.version)
         else:
             release_url = repo_url / 'releases/latest'
         async with self.manager.web_client.get(release_url) as response:
@@ -864,16 +917,16 @@ class GithubResolver(Resolver):
             description=project_metadata['description'],
             url=project_metadata['html_url'],
             download_url=matching_asset['browser_download_url'],
-            date_published=release_metadata['published_at'],
+            date_published=parse_datetime(release_metadata['published_at']),
             version=release_metadata['tag_name'],
-            options=m.PkgOptions(strategy=defn.strategy.type_.name),
+            options=m.PkgOptions(strategy=defn.strategy.name),
         )
 
 
 class InstawowResolver(Resolver):
     source = 'instawow'
     name = 'instawow'
-    strategies = {Strategies.default}
+    strategies = {Strategy.default}
 
     _addons = {
         ('0', 'weakauras-companion'),
@@ -881,6 +934,9 @@ class InstawowResolver(Resolver):
     }
 
     async def resolve_one(self, defn: Defn, metadata: Any) -> m.Pkg:
+        if defn.strategy not in self.strategies:
+            raise E.PkgStrategyUnsupported(defn.strategy)
+
         try:
             source_id, slug = next(p for p in self._addons if defn.alias in p)
         except StopIteration:
@@ -888,14 +944,10 @@ class InstawowResolver(Resolver):
 
         from .wa_updater import BuilderConfig, WaCompanionBuilder
 
-        sentinel = '__sentinel__'
-        builder = WaCompanionBuilder(self.manager, BuilderConfig(account=sentinel))
+        builder = WaCompanionBuilder(self.manager, BuilderConfig())
         if source_id == '1':
-            if builder.builder_config.account == sentinel:
-                raise E.PkgFileUnavailable('account name not provided')
             await builder.build()
 
-        checksum = await t(builder.checksum)()
         return m.Pkg(
             source=self.source,
             id=source_id,
@@ -904,7 +956,24 @@ class InstawowResolver(Resolver):
             description='A WeakAuras Companion clone.',
             url='https://github.com/layday/instawow',
             download_url=builder.addon_file.as_uri(),
-            date_published=datetime.now(),
-            version=checksum[:7],
-            options=m.PkgOptions(strategy=defn.strategy.type_.name),
+            date_published=datetime.now(timezone.utc),
+            version=(await t(builder.checksum)())[:7],
+            options=m.PkgOptions(strategy=defn.strategy.name),
+        )
+
+    @classmethod
+    async def collect_items(
+        cls, web_client: aiohttp.ClientSession
+    ) -> AsyncIterable[_CatalogueEntryDefaultFields]:
+        yield _CatalogueEntryDefaultFields(
+            source=cls.source,
+            id='1',
+            slug='weakauras-companion-autoupdate',
+            name='WeakAuras Companion',
+            folders=[
+                ['WeakAurasCompanion'],
+            ],
+            game_compatibility={'retail', 'classic'},
+            download_count=1,
+            last_updated=datetime.now(timezone.utc),
         )
