@@ -1,6 +1,8 @@
 import pytest
 from yarl import URL
 
+from instawow.models import is_pkg
+from instawow.resolvers import Defn
 from instawow.wa_updater import BuilderConfig, WaCompanionBuilder, WeakAura, WeakAuras
 
 
@@ -10,7 +12,7 @@ def wa_saved_vars(manager):
         manager.config.addon_dir.parents[1] / 'WTF' / 'Account' / 'test' / 'SavedVariables'
     )
     saved_vars.mkdir(parents=True)
-    (saved_vars / WeakAuras.filename).write_text(
+    (saved_vars / WeakAuras._filename).write_text(
         '''\
 WeakAurasSaved = {
     ["displays"] = {
@@ -25,7 +27,7 @@ WeakAurasSaved = {
 
 @pytest.fixture
 def builder(manager):
-    yield WaCompanionBuilder(manager, BuilderConfig(account='test'))
+    yield WaCompanionBuilder(manager, BuilderConfig())
 
 
 def test_can_parse_empty_displays_table(builder):
@@ -38,7 +40,7 @@ WeakAurasSaved = {
     },
 }
 ''',
-        ).entries
+        ).__root__
         == {}
     )
 
@@ -56,7 +58,7 @@ WeakAurasSaved = {
     },
 }
 ''',
-        ).entries
+        ).__root__
         == {}
     )
 
@@ -85,7 +87,7 @@ WeakAurasSaved = {
     },
 }
 ''',
-        ).entries
+        ).__root__
         == {'foo': [aura]}
     )
 
@@ -108,7 +110,7 @@ WeakAurasSaved = {
     },
 }
 ''',
-        ).entries
+        ).__root__
         == {}
     )
 
@@ -127,3 +129,18 @@ def test_build_is_reproducible(builder):
     checksum = builder.checksum()
     builder.make_addon([])
     assert checksum == builder.checksum()
+
+
+@pytest.mark.asyncio
+async def test_can_resolve_wa_companion_pkg(builder):
+    await builder.build()
+    defn = Defn('instawow', 'weakauras-companion')
+    resolve_results = await builder.manager.resolve([defn])
+    assert is_pkg(resolve_results[defn])
+
+
+@pytest.mark.asyncio
+async def test_can_resolve_wa_companion_autoupdate_pkg(monkeypatch, builder):
+    defn = Defn('instawow', 'weakauras-companion-autoupdate')
+    resolve_results = await builder.manager.resolve([defn])
+    assert is_pkg(resolve_results[defn])
